/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamrangefinderfork.h"
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>


static int forked_open_fd(GdamMidiDevice* midi_device)
{
	int pid;
	int fds[2];
	if (pipe(fds) < 0)
		g_error("creating pipe: %s", strerror(errno));
	pid = fork();
	if (pid < 0)
		g_error("error forking: %s", strerror(errno));
	if (pid == 0) {
		close(fds[0]);
		dup2(fds[1], STDOUT_FILENO);
		execl("/bin/sh", "sh", "-c", midi_device->filename, (char*)NULL);
		g_error("couldn't exec /bin/sh?");
	}
	close(fds[1]);
	return fds[0];
}

static void gdam_range_finder_fork_init(GdamRangeFinderFork* range_finder_fork)
{
}
static void gdam_range_finder_fork_class_init(GdamMidiDeviceClass* klass)
{
	klass->open_fd = forked_open_fd;
}

GtkType gdam_range_finder_fork_get_type()
{
	static GtkType range_finder_fork_type = 0;
	if (!range_finder_fork_type) {
		static const GtkTypeInfo range_finder_fork_info = {
			"GdamRangeFinderFork",
			sizeof(GdamRangeFinderFork),
			sizeof(GdamRangeFinderForkClass),
			(GtkClassInitFunc) gdam_range_finder_fork_class_init,
			(GtkObjectInitFunc) gdam_range_finder_fork_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent = GDAM_TYPE_RANGE_FINDER;
		range_finder_fork_type = gtk_type_unique(parent,
		                                 &range_finder_fork_info);
	}
	return range_finder_fork_type;
}
