/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
/* All-Pass Interpolating Delay Line filter
 *
 * Filtered to smooth out the noise that comes
 * with in linearly-interpolating the fractional
 * sample.
 *
 * Heavily based on "DLineA" in Perry R. Cook's STK.
 *           (Based on a version revised by Gary Scavone)
 */

#ifndef __GDAM_SYNTH_DELAY_LINE_ALL_PASS_H_
#define __GDAM_SYNTH_DELAY_LINE_ALL_PASS_H_

#include "gdamsynthdelayline.h"

typedef struct _GdamSynthDelayLineAllPass GdamSynthDelayLineAllPass;
typedef struct _GdamSynthDelayLineAllPassClass GdamSynthDelayLineAllPassClass;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdam_synth_delay_line_all_pass_get_type();
#define GDAM_TYPE_SYNTH_DELAY_LINE_ALL_PASS			(gdam_synth_delay_line_all_pass_get_type ())
#define GDAM_SYNTH_DELAY_LINE_ALL_PASS(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_SYNTH_DELAY_LINE_ALL_PASS, GdamSynthDelayLineAllPass))
#define GDAM_SYNTH_DELAY_LINE_ALL_PASS_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_SYNTH_DELAY_LINE_ALL_PASS, GdamSynthDelayLineAllPassClass))
#define GDAM_IS_SYNTH_DELAY_LINE_ALL_PASS(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_SYNTH_DELAY_LINE_ALL_PASS))
#define GDAM_IS_SYNTH_DELAY_LINE_ALL_PASS_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_SYNTH_DELAY_LINE_ALL_PASS))

struct _GdamSynthDelayLineAllPassClass {
	GdamSynthDelayLineClass	delay_line_class;
};
struct _GdamSynthDelayLineAllPass {
	GdamSynthDelayLine	synth_delay_line_all_pass;

	/* Retained for debugging after setting the delay, but
	 * unused otherwise... */
	gdouble			out_point_fraction;

	/* Coefficient for the allpass filter. */
	gdouble			coeff;

	/* For the allpass filter. */
	gdouble			last_in, last_out;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
