#include "gdambeatcalculatorskin.h"
#include <gdam/gdam-protocol.h>
#include "../gdamskinutil.h"
#include "../gdampacker.h"
#include "../gdamsongselectorskin.h"
#include <gtk/gtksignal.h>
#include <gdam/component/gdamsongdatabase.h>
#include "../../component/plugins/gdamproducermp3click.h"
#include <string.h>

#define SCALE_FACTOR	20

static GtkObjectClass* parent_class = NULL;

static void gdam_beat_calculator_skin_destroy(GtkObject* object);

static void gdam_beat_calculator_skin_class_init(GdamSkinClass* skin_class);
static void gdam_beat_calculator_skin_init(GdamBeatCalculatorSkin* beat_calculator_skin)
{
}

static void gdam_beat_calculator_post_connecting_handler(GdamSkin*      skin)
{
	GdamBufferEditorSkin* editor_skin = GDAM_BUFFER_EDITOR_SKIN(skin);
	GdamBufferViewport* viewport;
	g_return_if_fail(editor_skin != NULL);
	viewport = editor_skin->viewport;
	g_return_if_fail(viewport != NULL);
	viewport->use_buffer_manager = FALSE;
	viewport->num_beats = 100000;
	viewport->start_beat = 0.0;
	viewport->beat_period = 1.0;
	viewport->samples_per_second = 44100 / SCALE_FACTOR;
	viewport->is_selectable = TRUE;
	viewport->can_drag_beat_lines = TRUE;
}

GtkType gdam_beat_calculator_skin_get_type()
{
	static GtkType beat_calculator_skin_type = 0;
	if (!beat_calculator_skin_type) {
		static const GtkTypeInfo beat_calculator_skin_info = {
			"GdamBeatCalculatorSkin",
			sizeof(GdamBeatCalculatorSkin),
			sizeof(GdamBeatCalculatorSkinClass),
			(GtkClassInitFunc) gdam_beat_calculator_skin_class_init,
			(GtkObjectInitFunc) gdam_beat_calculator_skin_init,
			/* reserved_1*/ NULL,
			/* reserved_2*/ NULL,
			(GtkClassInitFunc) NULL
		};
		GtkType parent = GDAM_TYPE_BUFFER_EDITOR_SKIN;
		parent_class = gtk_type_class(parent);
		beat_calculator_skin_type = gtk_type_unique(parent,
		                                 &beat_calculator_skin_info);
	}
	return beat_calculator_skin_type;
}

/*  This is the last in a chain of callbacks used to create to buffer
 *  of sound data viewed onscreen.  It also handles various initializations,
 *  i.e. beat info. */
static void got_length(int num_args, GdamArg* args, gpointer user_data)
{
	GdamBeatCalculatorSkin* beat_calc;
	GdamBufferEditorSkin* editor_skin;
	GdamBufferViewport* viewport;
	gdouble length = 1.0;
	gdouble view_center = 0;
	beat_calc = GDAM_BEAT_CALCULATOR_SKIN(user_data);
	editor_skin = GDAM_BUFFER_EDITOR_SKIN(user_data);
	viewport = editor_skin->viewport;

	if (num_args < 1
	 || strcmp(args->name, "length") != 0
	 || args->type != GDAM_ARG_TYPE_INT64) {
	 	g_warning("couldn't get length???");
	} else {
		length = (gdouble)GDAM_ARG_INT64(args) / 44100.0 * SCALE_FACTOR;
		editor_skin->viewport->sample_length = length;
		viewport->time_length = length/2;
	}

	if (beat_calc->song_info != NULL) {
		GdamSongInfo* song_info = beat_calc->song_info;
		GdamBeatInfo* beat_info;
		beat_info = gdam_song_info_get_beat_info(song_info);
		if (beat_info != NULL) {
			gdouble start, bpm;
			viewport->num_beats = 100000;
			start = gdam_beat_info_get_first_beat(beat_info, 0.0);
			bpm = gdam_beat_info_get_bpm(beat_info, 0.0);
			view_center = start;
			viewport->start_beat = start;
			viewport->beat_period = 60.0 / bpm;
			gdam_model_set_beat_info(GDAM_SKIN(beat_calc)->model,
						 beat_info);
		} else {
			g_warning("song info contained no beat info");
		}
	} else {
		g_warning("no song info...");
	}

	beat_calc->is_caching = FALSE;
	g_return_if_fail(editor_skin->viewport != NULL);

	gdam_buffer_viewport_set_length(editor_skin->viewport, length);

	gdam_buffer_viewport_maybe_uncache(editor_skin->viewport);
	gdam_buffer_viewport_set_buffer_id(editor_skin->viewport,
	                                   beat_calc->buffer_id);
	gdam_buffer_viewport_whole_sample(editor_skin->viewport);
	gdam_buffer_viewport_set_center(editor_skin->viewport, view_center);
	gdam_buffer_viewport_set_selection(editor_skin->viewport, -1, -1);
	gdam_buffer_viewport_beat_lines_changed(editor_skin->viewport);
	gtk_object_unref(GTK_OBJECT(beat_calc));
	gdam_buffer_viewport_queue_redraw(editor_skin->viewport);
}

static void failed_to_get_length(gpointer user_data)
{
	GdamBeatCalculatorSkin* beat_calc;
	GdamBufferEditorSkin* editor_skin;
	beat_calc = GDAM_BEAT_CALCULATOR_SKIN(user_data);
	editor_skin = GDAM_BUFFER_EDITOR_SKIN(user_data);
	beat_calc->is_caching = FALSE;
	gtk_object_unref(GTK_OBJECT(beat_calc));
	g_warning("failed to get buffer length???");
	gdam_buffer_viewport_queue_redraw(editor_skin->viewport);
	gdam_buffer_viewport_maybe_uncache(editor_skin->viewport);
	gdam_buffer_viewport_set_buffer_id(editor_skin->viewport,
	                                   beat_calc->buffer_id);
	/* Set up beat info? */
}


static void finished_caching(gpointer user_data)
{
	GdamBeatCalculatorSkin* beat_calc;
	const char* length = "length";
	GdamBufferEditorSkin* editor_skin;
	beat_calc = GDAM_BEAT_CALCULATOR_SKIN(user_data);
	editor_skin = GDAM_BUFFER_EDITOR_SKIN(user_data);

	gdam_channel_append_request(
		GDAM_SKIN(user_data)->channel,
		gdam_request_get_gen_args(GDAM_PROTOCOL_BUFFER_ARG,
		                          beat_calc->buffer_id,
					  1,
					  &length,
					  got_length,
					  failed_to_get_length,
					  user_data)
	);
}

static void started_caching_at_server(int buffer_id,
                                      int event_id,
				      gpointer user_data)
{
	GdamBeatCalculatorSkin* beat_calc;
	GdamChannelEventHandler* event_handler;
	beat_calc = GDAM_BEAT_CALCULATOR_SKIN(user_data);
	beat_calc->buffer_id = buffer_id;
	beat_calc->cache_done_event_id = event_id;
	event_handler = gdam_channel_simple_handler_new(event_id,
							TRUE,
							finished_caching,
							beat_calc);
	gdam_channel_register_handler(GDAM_SKIN(beat_calc)->channel,
	                              event_handler);
}

static void failed_starting_to_cache(gpointer user_data)
{
	GdamBeatCalculatorSkin* beat_calc;
	beat_calc = GDAM_BEAT_CALCULATOR_SKIN(user_data);
	beat_calc->is_caching = FALSE;
	gtk_object_unref(GTK_OBJECT(beat_calc));
}

void gdam_beat_calculator_skin_load(GdamBeatCalculatorSkin* skin,
                                    GdamSongInfo*           song_info)
{
	GdamRequestBuilder* builder;
	int resample_id;
	int mp3_id;

	builder = gdam_request_builder_new();

	if (skin->is_caching) {
		/* TODO: stop the current caching song. */
		g_warning("Already caching song, aborting....");
		return;
	}
	gtk_object_ref(GTK_OBJECT(skin));
	mp3_id = gdam_request_builder_append_create(
			builder,
			gdam_request_create_source
				("GdamMp3",
				 NULL,
				 NULL,
				 NULL,
				 2,
				 "filename",
				 GDAM_ARG_TYPE_STRING,
				 gdam_song_info_get_filename(song_info),
				 "downsample",
				 GDAM_ARG_TYPE_INT,
				 4)
		);
	resample_id = gdam_request_builder_append_create(
			builder,
			gdam_request_create_source
				("GdamResample",
				 NULL,
				 NULL,
				 NULL,
				 2,
				 "filtered",
				 GDAM_ARG_TYPE_INT,
				 mp3_id,
				 "rate",
				 GDAM_ARG_TYPE_DOUBLE,
				 (gdouble) SCALE_FACTOR)
		);

	gdam_request_builder_append_generic(
			builder,
			gdam_request_start_caching
				(resample_id,
				 started_caching_at_server,
				 failed_starting_to_cache,
				 skin)
	);
	gdam_channel_append_request(
		GDAM_SKIN(skin)->channel,
		gdam_request_builder_package(builder)
	);
	gdam_request_builder_destroy(builder);

	skin->song_info = gdam_song_info_copy(song_info);
	skin->is_caching = TRUE;
}

void gdam_beat_calculator_skin_load_from_filename
                                   (GdamBeatCalculatorSkin* skin,
                                    const char*             filename)
{
	GdamSongInfo* song_info;
	GdamSongDatabase* database = gdam_song_database_system();
	GdamProducerMp3Click* mp3;
	gboolean must_free = FALSE;
	mp3 = GDAM_PRODUCER_MP3_CLICK(GDAM_SKIN(skin)->model);
	song_info = gdam_song_database_find(database, filename);
	if (song_info == NULL) {
		g_warning("no beat info for %s", filename);
		song_info = gdam_song_info_new(filename, 0.0, 0.0);
		must_free = TRUE;
	}
	gdam_beat_calculator_skin_load(skin, song_info);
	gdam_producer_mp3_click_set_filename(mp3, filename);
	if (must_free)
		gdam_song_info_destroy(song_info);
}

void gdam_beat_calculator_skin_beat_info_changed
                                   (GdamBeatCalculatorSkin* skin)
{
	GdamBufferViewport* viewport = GDAM_BUFFER_EDITOR_SKIN(skin)->viewport;
	if (viewport == NULL)
		return;
	gdam_buffer_viewport_queue_redraw(viewport);
}

static void stop_callback(GdamBeatCalculatorSkin* skin)
{
	GdamModel* model;
	model = GDAM_SKIN(skin)->model;
	if (model == NULL)
		return;
	gdam_producer_stop(GDAM_PRODUCER(model));
}

static void play_callback(GdamBeatCalculatorSkin* skin)
{
	GdamModel* mp3;
	GdamBufferViewport* viewport = GDAM_BUFFER_EDITOR_SKIN(skin)->viewport;
	GdamPlayInfo* play_info = g_new0(GdamPlayInfo,1);
	if (viewport == NULL)
		return;
	mp3 = GDAM_SKIN(skin)->model;
	if (mp3 == NULL)
		return;
	if (viewport->has_selection){
		play_info->position = viewport->selection_start;
		play_info->length = viewport->selection_length;
	}
	else{
		play_info->position = viewport->time_start;
		play_info->length = viewport->time_length;
	}
	gdam_producer_play(GDAM_PRODUCER(mp3), play_info);
	g_free(play_info);
}


static gboolean gdam_beat_calculator_preview_label_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	GdamBeatCalculatorSkin* beat_calc = GDAM_BEAT_CALCULATOR_SKIN(skin);
	beat_calc->preview_label = GTK_LABEL(object);
	return TRUE;
}

static gboolean gdam_beat_calculator_filename_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	GdamBeatCalculatorSkin* beat_calc = GDAM_BEAT_CALCULATOR_SKIN(skin);
	beat_calc->filename_entry = GTK_ENTRY(object);
	return TRUE;
}

static int delta_millis(GTimeVal* older, GTimeVal* younger)
{
	int millis = (older->tv_sec - younger->tv_sec) * 1000
	           + (older->tv_usec - younger->tv_usec) / 1000;
	return millis;
}

static void do_tap(GdamBeatCalculatorSkin* skin)
{
	GTimeVal current_time;
	int millis;
	gdouble del, beatlength, bpm;
	g_get_current_time(&current_time);
	if (!skin->have_first_tap){
		skin->first_tap_time = current_time;
		skin->have_first_tap = TRUE;
		return;
	}
	millis = delta_millis(&current_time, &skin->first_tap_time);
	skin->taps++;
	del = .001 * millis;
	beatlength = del / skin->taps;
	bpm = 60.0 / beatlength;
	gdam_debug("tapped bpm: %3.2f", bpm);
	if (skin->preview_label){
		char time[16];
		sprintf(time, "%3.2f", bpm);
		gtk_label_set_text(skin->preview_label, time);
	}
}

static void do_reset(GdamBeatCalculatorSkin* skin)
{
	skin->have_first_tap = FALSE;
	skin->taps = 0;
	if (skin->preview_label){
		gtk_label_set_text(skin->preview_label, "000.00");
	}
}

static void do_load(GdamBeatCalculatorSkin* beat_calc)
{
	char* filename;
	g_return_if_fail(beat_calc->filename_entry != NULL);
	filename = gtk_entry_get_text(beat_calc->filename_entry);
	g_return_if_fail(filename != NULL && filename[0] != '\0');

	gdam_beat_calculator_skin_load_from_filename(beat_calc, filename);
}

static gboolean gdam_beat_calculator_stop_button_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	gtk_signal_connect_object(object,
	                          signal_name, 
		                  GTK_SIGNAL_FUNC(stop_callback),
			          GTK_OBJECT(skin));
	return TRUE;
}

static gboolean gdam_beat_calculator_play_button_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	gtk_signal_connect_object(object,
	                          signal_name, 
		                  GTK_SIGNAL_FUNC(play_callback),
			          GTK_OBJECT(skin));
	return TRUE;
}

static gboolean gdam_beat_calculator_tap_button_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	gtk_signal_connect_object(object,
	                          signal_name, 
		                  GTK_SIGNAL_FUNC(do_tap),
			          GTK_OBJECT(skin));
	return TRUE;
}

static gboolean gdam_beat_calculator_reset_button_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	gtk_signal_connect_object(object,
	                          signal_name, 
		                  GTK_SIGNAL_FUNC(do_reset),
			          GTK_OBJECT(skin));
	return TRUE;
}

static gboolean gdam_beat_calculator_load_button_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	gtk_signal_connect_object(object,
	                          signal_name, 
		                  GTK_SIGNAL_FUNC(do_load),
			          GTK_OBJECT(skin));
	return TRUE;
}

static void bpm_adjusted(GtkAdjustment* adjustment,
                         GdamBeatCalculatorSkin* beat_calc_skin)
{
	GdamBufferViewport* viewport;
	viewport = GDAM_BUFFER_EDITOR_SKIN(beat_calc_skin)->viewport;
	g_return_if_fail(viewport != NULL);
	viewport->beat_period = 60.0 / adjustment->value;
	gdam_buffer_viewport_queue_redraw(viewport);
	gdam_buffer_viewport_beat_lines_changed(viewport);
}

static void on_viewport_beat_lines_changed(GdamBufferViewport* viewport,
					   GdamBeatCalculatorSkin* skin)
{
	GdamBeatInfo new_beat;
	if (viewport->beat_period <= 0)
		return;
	gdam_beat_info_set_bpm(&new_beat, 60.0 / viewport->beat_period);
	gdam_beat_info_set_first_beat(&new_beat, viewport->start_beat);
	gdam_model_set_beat_info(GDAM_SKIN(skin)->model, &new_beat);
}

static void bpm_adjustment_beat_lines_changed(GdamBufferViewport* viewport,
                                              GtkAdjustment* bpm_adj)
{
	gdouble bpm = 60.0 / viewport->beat_period;
	gtk_adjustment_set_value(bpm_adj, bpm);
}

static void on_filename_received(
			    const char* 	  filenames,
			    gpointer          data)
{
	GdamBeatCalculatorSkin* skin = GDAM_BEAT_CALCULATOR_SKIN(data);
	char filename[1024];
	char* nl;
	strncpy(filename, filenames, 1024);
	nl = strchr(filename, '\n');
	if (nl)
		*nl = '\0';
	gdam_debug("beat_calc: filename %s dropped", filename);
	if (skin->filename_entry != NULL)
		gtk_entry_set_text(skin->filename_entry, filename);
	gdam_beat_calculator_skin_load_from_filename(skin, filename);
}

static void do_import_files(GdamBeatCalculatorSkin* skin)
{
	GdamSongSelectorSkin* songselskin;
	GdamSkin* packer;
	char filename[1024];
	char* filenames;
	char* nl;

	packer = gdam_skin_try_get_unique("GdamSongSelector");
	if (!packer){
		gdam_debug("producermp3skin: no songsel open, can't import.");
		return;
	}
	songselskin = GDAM_SONG_SELECTOR_SKIN( 
			gdam_packer_get_skin(GDAM_PACKER(packer), "songsel"));
	filenames = gdam_song_selector_export_selection_string(songselskin);
	strncpy(filename, filenames, 1024);
	nl = strchr(filename, '\n');
	if (nl)
		*nl = '\0';

	gdam_debug("beat_calc: filename %s imported", filename);
	gdam_beat_calculator_skin_load_from_filename(skin, filename);

	return;
}


static gboolean gdam_beat_calculator_import_songsel_files_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	gtk_signal_connect_object(object,
			   	  signal_name,
			   	  GTK_SIGNAL_FUNC(do_import_files),
			   	  GTK_OBJECT(skin));
	return TRUE;
}


static gboolean gdam_beat_calculator_filename_dropped_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	gdam_widget_string_dest_set(GTK_WIDGET(object),
				    on_filename_received,
				    "gdam-filename-text",
				    skin);
	return TRUE;
}

static gboolean gdam_beat_calculator_bpm_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	GdamBeatCalculatorSkin* beat_calc = GDAM_BEAT_CALCULATOR_SKIN(skin);
	GtkWidget* widget = GTK_WIDGET(object);
	GtkAdjustment* adjustment = util_get_widget_adjustment(widget);
	GdamBufferViewport* viewport;
	viewport = GDAM_BUFFER_EDITOR_SKIN(beat_calc)->viewport;
	if (!viewport)
		return FALSE;
	beat_calc->bpm_adj = adjustment;
	gtk_signal_connect(GTK_OBJECT(adjustment),
	                   "value_changed",
			   GTK_SIGNAL_FUNC(bpm_adjusted),
			   skin);
	gtk_signal_connect(GTK_OBJECT(viewport),
	                   "beat_lines_changed",
			   GTK_SIGNAL_FUNC(bpm_adjustment_beat_lines_changed),
			   adjustment);
	return TRUE;
}

static void start_beat_adjusted(GtkAdjustment* adjustment,
                         GdamBeatCalculatorSkin* beat_calc_skin)
{
	GdamBufferViewport* viewport;
	viewport = GDAM_BUFFER_EDITOR_SKIN(beat_calc_skin)->viewport;
	g_return_if_fail(viewport != NULL);
	viewport->start_beat = adjustment->value;
	gdam_buffer_viewport_queue_redraw(viewport);
	gdam_buffer_viewport_beat_lines_changed(viewport);
}

static void start_beat_adjustment_beat_lines_changed
                                             (GdamBufferViewport* viewport,
                                              GtkAdjustment* bpm_adj)
{
	gtk_adjustment_set_value(bpm_adj, viewport->start_beat);
}


static gboolean gdam_beat_calculator_start_beat_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	GdamBeatCalculatorSkin* beat_calc = GDAM_BEAT_CALCULATOR_SKIN(skin);
	GtkWidget* widget = GTK_WIDGET(object);
	GtkAdjustment* adjustment = util_get_widget_adjustment(widget);
	GdamBufferViewport* viewport;
	viewport = GDAM_BUFFER_EDITOR_SKIN(beat_calc)->viewport;
	if (!viewport)
		return FALSE;

	beat_calc->start_beat_adj = adjustment;
	gtk_signal_connect(GTK_OBJECT(adjustment),
	                   "value_changed",
			   GTK_SIGNAL_FUNC(start_beat_adjusted),
			   skin);
	gtk_signal_connect(GTK_OBJECT(viewport),
	                   "beat_lines_changed",
		   GTK_SIGNAL_FUNC(start_beat_adjustment_beat_lines_changed),
			   adjustment);
	return TRUE;
}


static void do_set_in_database(GdamSkin* skin)
{
	GdamBeatCalculatorSkin* beat_calc = GDAM_BEAT_CALCULATOR_SKIN(skin);
	GdamSongInfo* song_info = beat_calc->song_info;
	GdamBufferViewport* viewport = GDAM_BUFFER_EDITOR_SKIN(skin)->viewport;
	GdamSongDatabase* database = gdam_song_database_system();
	g_return_if_fail(viewport != NULL);
	g_return_if_fail(song_info != NULL);
	gdam_song_info_set_bpm(song_info, 60.0 / viewport->beat_period);
	gdam_song_info_set_first_beat(song_info, viewport->start_beat);
	gdam_song_info_add_property(song_info, "synch", "bc");
	gtk_signal_emit_by_name(GTK_OBJECT(database), "changed");
	gdam_debug("do_set_in_database: bpm=%4.4g, start_beat=%4.4g",
		60.0 / viewport->beat_period, viewport->start_beat);
}

static gboolean gdam_beat_calculator_set_in_database_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
	gtk_signal_connect_object(object,
	                          signal_name,
	                          GTK_SIGNAL_FUNC(do_set_in_database),
				  GTK_OBJECT(skin));
	return TRUE;
}

static GdamSkinGenericConnect connect_infos[] =
{
	{ "gdam_beat_calculator_filename",
	  gdam_beat_calculator_filename_handler },
	{ "gdam_beat_calculator_load_button",
	  gdam_beat_calculator_load_button_handler },
	{ "gdam_beat_calculator_tap_button",
	  gdam_beat_calculator_tap_button_handler },
	{ "gdam_beat_calculator_reset_button",
	  gdam_beat_calculator_reset_button_handler },
	{ "gdam_beat_calculator_preview_label",
	  gdam_beat_calculator_preview_label_handler },
	{ "gdam_beat_calculator_play_button",
	  gdam_beat_calculator_play_button_handler },
	{ "gdam_beat_calculator_stop_button",
	  gdam_beat_calculator_stop_button_handler },
	{ "gdam_beat_calculator_bpm_adjustment",
	  gdam_beat_calculator_bpm_adjustment_handler },
	{ "gdam_beat_calculator_filename_dropped",
	  gdam_beat_calculator_filename_dropped_handler },
	{ "gdam_beat_calculator_import_songsel_files",
	   gdam_beat_calculator_import_songsel_files_handler	},
	{ "gdam_beat_calculator_start_beat_adjustment",
	  gdam_beat_calculator_start_beat_adjustment_handler },
	{ "gdam_beat_calculator_set_in_database",
	  gdam_beat_calculator_set_in_database_handler }
	  };


static gboolean       gdam_beat_calculator_build_new_model
                                                 (GdamSkin*      skin,
                                                  GdamChannel*   channel,
						  int		 num_options,
						  char**         options)
{
	GdamModel* model;
	GdamBufferEditorSkin* editor = GDAM_BUFFER_EDITOR_SKIN(skin);
	GdamBufferViewport* viewport;
	GdamProducerMp3Click* producer_mp3 = gtk_type_new(GDAM_TYPE_PRODUCER_MP3_CLICK);
	producer_mp3->non_seekable = TRUE;
	model = GDAM_MODEL(producer_mp3);
	//GDAM_PRODUCER_MP3(model)->non_seekable = TRUE;
	model->channel = channel;
	skin->model = model;
	viewport = gdam_buffer_viewport_new_shown(channel);
	gtk_object_ref(GTK_OBJECT(viewport));
	gtk_object_sink(GTK_OBJECT(viewport));
	editor->viewport = viewport;
	gtk_signal_connect(GTK_OBJECT(viewport),
	                   "beat_lines_changed",
			   GTK_SIGNAL_FUNC(on_viewport_beat_lines_changed),
			   skin);

	return TRUE;
}

static void gdam_beat_calculator_skin_class_init(GdamSkinClass* skin_class)
{
	skin_class->build_new_model = 
				gdam_beat_calculator_build_new_model;
	skin_class->post_connecting_handler
			= gdam_beat_calculator_post_connecting_handler;
	GTK_OBJECT_CLASS(skin_class)->destroy
			= gdam_beat_calculator_skin_destroy;
	gdam_skin_class_register_generic_handlers(skin_class,
				                  G_N_ELEMENTS(connect_infos),
				                  connect_infos);
}

static void gdam_beat_calculator_skin_destroy(GtkObject* object)
{
	GdamBufferViewport* viewport;
	viewport = GDAM_BUFFER_EDITOR_SKIN(object)->viewport;
	if (viewport != NULL)
	 	gdam_buffer_viewport_maybe_uncache(viewport);
	(*parent_class->destroy)(object);
}
